﻿using System;
using System.Collections.Generic;
using Amplitude.AI.Core.AIPlayers.MajorEmpire;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Brain.Strategic.Data.Expansion;
using Amplitude.AI.Core.Brain.Tasks.MotherShipMissionTasks;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots.EntitySnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Galaxy;
using Amplitude.AI.Core.Snapshots.Position;

namespace Amplitude.AI.Core.Brain.Generators.Motherships
{
    public class HarvestBestSystem : ContextBasedGenerator<FleetSnapshot, MothershipHarvest, HarvestSystem, HarvestBestSystem.HarvestGenerationData>
    {
        public override string Description
        {
            get
            {
                return "Vampirilis have to manage mothership!";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 0;
            }
        }

        public override string LocalizedName
        {
            get
            {
                return "Harvest best system.";
            }
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            if (!aiPlayer.ControlledEmpireSnapshot.HasMotherships)
            {
                return;
            }
            for (int i = 0; i < aiPlayer.ControlledEmpireSnapshot.Fleets.Count; i++)
            {
                FleetSnapshot fleetSnapshot = aiPlayer.ControlledEmpireSnapshot.Fleets[i];
                if (fleetSnapshot.HasMothership)
                {
                    base.ContinueDecision(aiPlayer, fleetSnapshot);
                }
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, FleetSnapshot context)
        {
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<MothershipHarvest>());
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, HarvestBestSystem.HarvestGenerationData generationData)
        {
            HeuristicFloat result = default(HeuristicFloat);
            result.Add(generationData.HarvestGain, "Harvest gain");
            return result;
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, FleetSnapshot context, MothershipHarvest parent)
        {
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            HarvestSystem harvestSystem = aiPlayer.Blackboard.FindFirstRunningMotership<HarvestSystem>(context.GameEntityGuid);
            if (harvestSystem == null)
            {
                harvestSystem = new HarvestSystem(aiPlayer.Blackboard.GenerateGUID());
                harvestSystem.ContextGuid = aiPlayer.ControlledEmpireSnapshot.GameEntityGuid;
                harvestSystem.MotherFleet = context.GameEntityGuid;
                aiPlayer.Blackboard.Register<HarvestSystem>(harvestSystem);
            }
            NodePosition invalid = NodePosition.Invalid;
            HeuristicFloat gain = new HeuristicFloat(float.MinValue, "");
            ConstellationSnapshot constellationSnapshot;
            if (service.GalaxySnapshot.TryGetNodeConstellation(context.PreviousValidNodePosition, out constellationSnapshot))
            {
                this.ComputeBestNode(aiPlayer, context, ref invalid, ref gain, constellationSnapshot.GameNodes);
            }
            if (invalid == NodePosition.Invalid)
            {
                this.ComputeBestNode(aiPlayer, context, ref invalid, ref gain, service.GalaxySnapshot.GameNodes);
            }
            if (invalid == NodePosition.Invalid)
            {
                return;
            }
            harvestSystem.TargetNode = invalid;
            harvestSystem.MustDetach = false;
            base.ContinueDecision(aiPlayer, this.GetGenerationData(context, parent, harvestSystem, gain));
        }

        protected HarvestBestSystem.HarvestGenerationData GetGenerationData(FleetSnapshot context, MothershipHarvest parent, HarvestSystem message, HeuristicFloat gain)
        {
            return new HarvestBestSystem.HarvestGenerationData
            {
                Context = context,
                Parent = parent,
                Message = message,
                HarvestGain = gain
            };
        }

        private void ComputeBestNode(AIPlayer aiPlayer, FleetSnapshot context, ref NodePosition targetNode, ref HeuristicFloat targetGain, GameNodeSnapshot[] nodes)
        {
            for (int i = 0; i < nodes.Length; i++)
            {
                StarSystemNodeSnapshot starSystemNodeSnapshot = nodes[i] as StarSystemNodeSnapshot;
                if (starSystemNodeSnapshot != null)
                {
                    bool flag = context.HasMothershipAttached && starSystemNodeSnapshot.NodePosition == context.NodePosition;
                    if (flag || (!starSystemNodeSnapshot.HasColony(true, aiPlayer.ControlledEmpireIndex, true) && starSystemNodeSnapshot.Visibility.VisibilityFor(aiPlayer.ControlledEmpireIndex) != EntityVisibility.Layer.Unknown))
                    {
                        NodeData analysisData = starSystemNodeSnapshot.GetAnalysisData<NodeData>(aiPlayer);
                        if ((flag || starSystemNodeSnapshot.IsColonizable(aiPlayer.ControlledEmpireSnapshot as MajorEmpireSnapshot)) && !(analysisData.ColonizablePlanetCount == 0f) && !float.IsInfinity(analysisData.Distances[context.FleetPosition.NextValidNodePosition.NodeIndex]))
                        {
                            bool flag2 = false;
                            List<HarvestSystem> list = new List<HarvestSystem>();
                            aiPlayer.Blackboard.FindAllRunning<HarvestSystem>(ref list);
                            foreach (HarvestSystem harvestSystem in list)
                            {
                                if (harvestSystem.TargetNode.NodeIndex == starSystemNodeSnapshot.NodePosition.NodeIndex && harvestSystem.MotherFleet != context.GameEntityGuid)
                                {
                                    flag2 = true;
                                }
                            }
                            if (!flag2)
                            {
                                HeuristicFloat harvestGain = this.GetHarvestGain(context, analysisData);
                                if (flag && starSystemNodeSnapshot.NodePosition == context.NodePosition)
                                {
                                    harvestGain.Multiply(2f, "i'm allready attached to this system");
                                }
                                if (harvestGain > targetGain)
                                {
                                    targetGain = harvestGain;
                                    targetNode = starSystemNodeSnapshot.NodePosition;
                                }
                            }
                        }
                    }
                }
            }
        }

        private HeuristicFloat GetHarvestGain(FleetSnapshot context, NodeData nodeData)
        {
            HeuristicFloat result = default(HeuristicFloat);
            ExpansionDataHolder expansionDataHolder = (context.EmpireOwner.AIPlayer as AIPlayer_MajorEmpire).StrategicDataHolder.ExpansionDataHolder;
            result.Add(nodeData.ColonizationScore, "System gain");
            HeuristicFloat maxSystemColonizationScore = context.EmpireOwner.AIPlayer.AnalysisDataHolder.MyEmpireData.MaxSystemColonizationScore;
            if (maxSystemColonizationScore != 0f)
            {
                result.Divide(maxSystemColonizationScore, "Global systems gains");
            }
            result.Clamp01();
            HeuristicFloat operand = default(HeuristicFloat);
            HeuristicFloat operand2 = default(HeuristicFloat);
            HeuristicFloat heuristicFloat = this.ComputeDistanceFactor(context, nodeData);
            if (float.IsInfinity(heuristicFloat))
            {
                return new HeuristicFloat(float.NegativeInfinity, "");
            }
            operand2.Add(1f, "Constant");
            operand2.Subtract(heuristicFloat, "Distance factor");
            operand2.Clamp01();
            operand.Add(operand2, "1 - DistanceFactor");
            operand.Multiply(0.5f, "Distance score modifier (constant)");
            result.Multiply(operand, "Distance Score");
            return result;
        }

        private HeuristicFloat ComputeDistanceFactor(FleetSnapshot context, NodeData nodeData)
        {
            float num = nodeData.Distances[context.FleetPosition.NextValidNodePosition.NodeIndex];
            if (float.IsInfinity(num))
            {
                return new HeuristicFloat(float.PositiveInfinity, "");
            }
            HeuristicFloat result = default(HeuristicFloat);
            result.Add(num, "Path cost");
            result.Divide(20f, "Estimate Fleet movement per turn");
            result.Divide(5f, "Max 5 turns distance");
            if (num != 0f)
            {
                result.Add(0.5f, "constant to avoid ocillation when we have to move");
            }
            return result;
        }

        public HarvestBestSystem()
        {
        }

        public class HarvestGenerationData : GenerationData<FleetSnapshot, MothershipHarvest, HarvestSystem>
        {
            public HarvestGenerationData()
            {
            }

            public HeuristicFloat HarvestGain { get; set; }
        }
    }
}
